#########################################################
#
# Cosmic ray removal for Demetra
# The LA cosmic algorithm is called from exisiting Python implementation and wrapped
# in a suitable way for Demetra file naming.
#
# You may need to fix teh camera gain below which are currently for an Atik460EX
#
# Kevin Gurney 2021
#
#######################################################

import lacosmic
import sys
from astropy.io import fits
import shutil
import os
import re
from copy import copy as cp

######################################
# Atik460EX parameters
camera_gain = 0.27
camera_readnoise = 5
######################################

################################
### Other LA cosmic paramters

contrast = 3 #  lower/higher values for  higher/lower pixel density 

# for these see http://www.astro.wisc.edu/~cigan/reducing/lacos/lacos.html
nbr_threshold = 0.5   #0.5
#################################




############
cr_default = 55 # MILES short exposures 2x2
                # for longer exposure

if len(sys.argv) == 2: 
    cr_threshold = float(sys.argv[1])
else:
    cr_threshold = cr_default
    
print('using a threshold of ', cr_threshold, '\n')





#############
# Some fits file handling to recretae old header in cleaned up file

def make_header(new_h, org_h):
    org_keys = list(org_h.keys()) 
    new_keys = list(new_h.keys())
    for item in org_keys:
        if not(item in new_keys):
            if not(item == 'HISTORY'):
                val = org_h[item]
                new_h[item] = val
    return new_h

################
# main code
################

BAK_pat = '^BAK'
map_pat = '^map'
E_pat = '.*Object_[1-9]+_P.*'

found_first = False
for fname in os.listdir('.'):
    if (re.match(BAK_pat, fname) == None) and (re.match(map_pat, fname) == None) and (re.match(E_pat, fname) != None):
        print(fname)
        E_image_file = fname
        E_image_file_back = 'BAK_' + fname
        
        with fits.open(E_image_file) as hdu_org:
            data = cp(hdu_org[0].data) #use copy oterwise problems clsoing file 
            cleaned_image = lacosmic.lacosmic(data, contrast, cr_threshold, nbr_threshold, effective_gain=camera_gain, readnoise=camera_readnoise)
        
            hdu_image = fits.PrimaryHDU(cleaned_image[0])  
            hdu1 = fits.HDUList([hdu_image])
            new_hdr = hdu1[0].header
            org_hdr = hdu_org[0].header
            make_header(new_hdr, org_hdr)
        
        hdu1.close()
        hdu_org.close()
        #os.rename(E_image_file, E_image_file_back)
        shutil.move(E_image_file, E_image_file_back)
        hdu1.writeto(E_image_file)
        
        hdu_map = cleaned_image[1]
        if not found_first:
            cum_map = hdu_map.astype(int)
            found_first = True
        else:
            cum_map = cum_map + hdu_map.astype(int)
            
        m = hdu_map.astype(int)
        fmap = fits.PrimaryHDU(m)
        
        new_map_file = 'map_' + fname
        # full_new_map = target_folder + '\\' + new_map_file
        try:
            os.remove(new_map_file)
        except OSError as e:
            print('no map file to remove')
        
        fmap.writeto(new_map_file)

######
cum_map_file = 'full_map.fits'
try:
    os.remove(cum_map_file)
except OSError as e:
    print('no fixed  file to remove')
cum_fits  = fits.PrimaryHDU(cum_map)
cum_fits.writeto(cum_map_file)

ans = input('press \'Return\' to finish  \n')


        

